 /**
 ********************************************************************************
 * Copyright (c) 2017 - 2018 Robert Bosch GmbH.
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Robert Bosch GmbH - initial API and implementation
 ********************************************************************************
 */
package org.eclipse.app4mc.emfutils.content.extractor.builders;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.BasicEMap;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;

public class EClassHierarchyBufferBuilder {

	private EMap<String, Object> id2ObjectsMap;

	public EMap<String, Object> getId2ObjectsMap() {
		return this.id2ObjectsMap;
	}


	public void setId2ObjectsMap(final EMap<String, Object> id2ObjectsMap) {
		this.id2ObjectsMap = id2ObjectsMap;
	}


	public StringBuffer buildBuffer(final EClass selectedObjClass) {

		this.id2ObjectsMap = new BasicEMap<String, Object>();

		final StringBuffer plantumlBuffer = new StringBuffer();


		plantumlBuffer.append("@startuml");

		addClassInfo(selectedObjClass, plantumlBuffer);

		final EList<EClass> eSuperTypes = selectedObjClass.getESuperTypes();
		final List<EClass> subClasses = getSubClasses(selectedObjClass);

		if (eSuperTypes.size() > 0) {

			populateHeirarchy(plantumlBuffer, selectedObjClass, this.id2ObjectsMap);

		}

		if (subClasses.size() > 0) {

			/*- populating sub-classes */
			populateSubClasses(plantumlBuffer, subClasses, selectedObjClass, this.id2ObjectsMap);
		}

		plantumlBuffer.append(System.getProperty("line.separator"));

		plantumlBuffer.append("@enduml");

		return plantumlBuffer;

	}

	private void addClassInfo(final EClass selectedObjClass, final StringBuffer plantumlBuffer) {

		plantumlBuffer.append(System.getProperty("line.separator"));

		if (selectedObjClass.isAbstract() && selectedObjClass.isInterface()) {
			plantumlBuffer.append("interface \"" + selectedObjClass.getName() + "\"");

		}
		else if (selectedObjClass.isAbstract()) {
			plantumlBuffer.append("abstract \"" + selectedObjClass.getName() + "\"");

		}
		else if (selectedObjClass.isInterface()) {

			plantumlBuffer.append("interface \"" + selectedObjClass.getName() + "\"");
		}
		else {
			plantumlBuffer.append("class \"" + selectedObjClass.getName() + "\"");

		}

		plantumlBuffer.append(System.getProperty("line.separator"));
	}


	private void populateSubClasses(final StringBuffer plantumlBuffer, final List<EClass> subClasses,
			final EClass selectedObjClass, final EMap<String, Object> id2ObjectsMap) {

		for (final EClass subClass : subClasses) {

			final String uuid = EcoreUtil.generateUUID();


			if (subClass.isAbstract() && subClass.isInterface()) {
				plantumlBuffer.append("interface \"" + subClass.getName() + "\"" + "    [[" + uuid + "]]");

			}
			else if (subClass.isAbstract()) {
				plantumlBuffer.append("abstract \"" + subClass.getName() + "\"" + "    [[" + uuid + "]]");

			}
			else if (subClass.isInterface()) {

				plantumlBuffer.append("interface \"" + subClass.getName() + "\"" + "    [[" + uuid + "]]");
			}
			else {
				plantumlBuffer.append("class \"" + subClass.getName() + "\"" + "    [[" + uuid + "]]");

			}

			id2ObjectsMap.put(uuid, subClass);

			plantumlBuffer.append(System.getProperty("line.separator"));

			plantumlBuffer.append(selectedObjClass.getName() + " <|-- " + subClass.getName());

			plantumlBuffer.append(System.getProperty("line.separator"));


		}

	}

	private List<EClass> getSubClasses(final EClass selectedObjClass) {

		final List<EClass> allSubClasses = new ArrayList<EClass>();

		final ResourceSet resourceSet = selectedObjClass.eResource().getResourceSet();

		if (resourceSet != null) {

			// With this approach check is made to find the Sub-Classes for the supplied EClass in all the packages
			// belonging to the resourceset

			final EList<Resource> resources = resourceSet.getResources();

			for (final Resource resource : resources) {

				final EList<EObject> contents = resource.getContents();

				if (contents.size() > 0) {
//					final EObject eObject = contents.get(0);
					
					for (EObject eobj : contents) {
						
						if (eobj instanceof EPackage) {
							allSubClasses.addAll(getSubClasses(selectedObjClass, (EPackage) eobj));
						}
					}

				}

			}
		}
		else {

			final EPackage ePackage = selectedObjClass.getEPackage();
			return getSubClasses(selectedObjClass, ePackage);
		}

		return allSubClasses;

	}

	private List<EClass> getSubClasses(final EClass selectedObjClass, final EPackage ePackage) {
		final List<EClass> allSubClasses = new ArrayList<EClass>();

		final EList<EClassifier> eClassifiers = ePackage.getEClassifiers();

		for (final EClassifier eClassifier : eClassifiers) {

			if (eClassifier instanceof EClass) {

				if (!(selectedObjClass == eClassifier) && selectedObjClass.isSuperTypeOf((EClass) eClassifier)) {
					allSubClasses.add((EClass) eClassifier);
				}


			}
		}
		return allSubClasses;
	}


	private void populateHeirarchy(final StringBuffer plantumlBuffer, final EClass selectedObjClass,
			final EMap<String, Object> id2ObjectsMap) {
		for (final EClass superClass : selectedObjClass.getESuperTypes()) {

			final String uuid = EcoreUtil.generateUUID();

			if (superClass.isAbstract() && superClass.isInterface()) {
				plantumlBuffer.append("interface \"" + superClass.getName() + "\"" + "    [[" + uuid + "]]");

			}
			else if (superClass.isAbstract()) {
				plantumlBuffer.append("abstract \"" + superClass.getName() + "\"" + "    [[" + uuid + "]]");

			}
			else if (superClass.isInterface()) {

				plantumlBuffer.append("interface \"" + superClass.getName() + "\"" + "    [[" + uuid + "]]");
			}
			else {
				plantumlBuffer.append("class \"" + superClass.getName() + "\"" + "    [[" + uuid + "]]");

			}

			id2ObjectsMap.put(uuid, superClass);

			plantumlBuffer.append(System.getProperty("line.separator"));

			plantumlBuffer.append(superClass.getName() + " <|-- " + selectedObjClass.getName());

			plantumlBuffer.append(System.getProperty("line.separator"));

			populateHeirarchy(plantumlBuffer, superClass, id2ObjectsMap);
		}
	}


}
