 /**
 ********************************************************************************
 * Copyright (c) 2017 - 2018 Robert Bosch GmbH.
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Robert Bosch GmbH - initial API and implementation
 ********************************************************************************
 */
package org.eclipse.app4mc.emfutils.content.extractor.builders;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import org.eclipse.emf.common.util.BasicEMap;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;


public class ObjectContentBufferBuilder {

	/* EClassContents and EObjectRefs behaviour is included in ObjectContentBufferBuilder java class */

	private EMap<String, Object> id2ObjectsMap;

	public StringBuffer buildBuffer(final EObject eObject) {

		this.id2ObjectsMap = new BasicEMap<String, Object>();

		final StringBuffer plantumlBuffer = new StringBuffer();


		plantumlBuffer.append("@startuml");
		plantumlBuffer.append(System.getProperty("line.separator"));

		buildBuffer(eObject, plantumlBuffer);

		plantumlBuffer.append(System.getProperty("line.separator"));
		plantumlBuffer.append("@enduml");

		return plantumlBuffer;

	}


	public StringBuffer buildBufferForCollection(final EList<?> eList) {

		this.id2ObjectsMap = new BasicEMap<String, Object>();

		final StringBuffer plantumlBuffer = new StringBuffer();


		plantumlBuffer.append("@startuml");
		plantumlBuffer.append(System.getProperty("line.separator"));


		createClass(plantumlBuffer, this.id2ObjectsMap, eList, null);

		for (final Object object : getElementsBasedOnRange(eList)) {
			if (object instanceof EObject) {

				createClass(plantumlBuffer, this.id2ObjectsMap, (EObject) object);

				final String name = getName((EObject) object).toString();

				plantumlBuffer.append("Collection --> " + name);

				plantumlBuffer.append(System.getProperty("line.separator"));

			}
		}

		plantumlBuffer.append(System.getProperty("line.separator"));
		plantumlBuffer.append("@enduml");

		return plantumlBuffer;

	}


	public EMap<String, Object> getId2ObjectsMap() {
		return this.id2ObjectsMap;
	}


	private void buildBuffer(final EObject selectedEObj, final StringBuffer plantumlBuffer) {


		final EList<EStructuralFeature> eStructuralFeatures = selectedEObj.eClass().getEAllStructuralFeatures();


		/*- if refs of the EClass are found, then fill the plantUMLBuffer with the appropriate contents */

		if (eStructuralFeatures.size() > 0) {

			createClass(plantumlBuffer, this.id2ObjectsMap, selectedEObj);

			final List<EStructuralFeature> processedFeatures = new ArrayList<EStructuralFeature>();


			for (final EStructuralFeature eStructuralFeature : eStructuralFeatures) {

				final boolean eIsSet = selectedEObj.eIsSet(eStructuralFeature);

				if (eIsSet && !processedFeatures.contains(eStructuralFeature)
						&& (eStructuralFeature instanceof EReference)) {

					processedFeatures.add(eStructuralFeature);

					final Object eobj = selectedEObj.eGet(eStructuralFeature);


					if (eobj instanceof EObject) {
						createClass(plantumlBuffer, this.id2ObjectsMap, (EObject) eobj);

						final String name = getName((EObject) eobj).toString();

						final String relation = ((EReference) eStructuralFeature).isContainment() ? " *-- " : "--> ";

						plantumlBuffer.append(getName(selectedEObj) + relation + name);

						plantumlBuffer.append(System.getProperty("line.separator"));


					}
					else if (eobj instanceof EList) {

						createClass(plantumlBuffer, this.id2ObjectsMap, (EList<?>) eobj, eStructuralFeature);

						final String relation = ((EReference) eStructuralFeature).isContainment() ? " *-- " : "--> ";

						plantumlBuffer.append(getName(selectedEObj) + relation + eStructuralFeature.getName());

						plantumlBuffer.append(System.getProperty("line.separator"));


					}

				}
			}


		}
		else {

			/*- no references for this EClass are found */

			createClass(plantumlBuffer, this.id2ObjectsMap, selectedEObj);

		}


	}


	private void createClass(final StringBuffer plantumlBuffer, final EMap<String, Object> id2ObjectsMap,
			final EList<?> eList, final EStructuralFeature eStructuralFeature) {

		final String uuid = EcoreUtil.generateUUID();

		if (eStructuralFeature != null) {
			plantumlBuffer.append("class \"" + eStructuralFeature.getName() + "\"" + " << (*,#FF7700) EList<"
					+ eStructuralFeature.getEType().getName() + "> >>" + "    [[" + uuid + "]]");

		}
		else {
			plantumlBuffer.append("class \"" + "Collection" + "\"" + " << (*,#FF7700) Collection<" + "EList" + "> >>"
					+ "    [[" + uuid + "]]");
		}

		id2ObjectsMap.put(uuid, eList);

		plantumlBuffer.append(System.getProperty("line.separator"));
	}


	private void createClass(final StringBuffer plantumlBuffer, final EMap<String, Object> id2ObjectsMap,
			final EObject selectedEObj) {
		final String uuid = EcoreUtil.generateUUID();

		plantumlBuffer.append("class " + getName(selectedEObj) + " <<  (O,#B4A7E5) " + selectedEObj.eClass().getName()
				+ " >>" + "    [[" + uuid + "]]");

		final EList<EAttribute> eAllAttributes = selectedEObj.eClass().getEAllAttributes();

		if (eAllAttributes.size() > 0) {

			plantumlBuffer.append("{");

			plantumlBuffer.append(System.getProperty("line.separator"));

			final List<EAttribute> processedFeatures = new ArrayList<EAttribute>();

			for (final EAttribute eAttribute : eAllAttributes) {

				if (selectedEObj.eIsSet(eAttribute) && !processedFeatures.contains(eAttribute)) {

					processedFeatures.add(eAttribute);

					plantumlBuffer.append(System.getProperty("line.separator"));

					plantumlBuffer.append(eAttribute.getEType().getName() + " " + eAttribute.getName() + " = "
							+ selectedEObj.eGet(eAttribute));

					plantumlBuffer.append(System.getProperty("line.separator"));
				}


			}
			plantumlBuffer.append(System.getProperty("line.separator"));

			plantumlBuffer.append("}");
		}

		id2ObjectsMap.put(uuid, selectedEObj);

		plantumlBuffer.append(System.getProperty("line.separator"));
	}


	private Object getName(final EObject eObject) {
		final EStructuralFeature eStructuralFeature = eObject.eClass().getEStructuralFeature("name");

		if (eStructuralFeature != null) {

			final Object originalName = eObject.eGet(eStructuralFeature);

			return "\"" + originalName + "\"";
		}

		return eObject.eClass().getName() + "__" + eObject.hashCode();
	}

	private List<?> getElementsBasedOnRange(final EList<?> eList) {


		if (eList.size() > 100) {

			List<?> subList = new ArrayList();

			final InputDialog inputDialog = new InputDialog(
					org.eclipse.swt.widgets.Display.getDefault().getActiveShell(), "Element selection dialog",
					"There are " + eList.size()
							+ " references. specify the range for which contents should be represented graphically",
					"0-" + (eList.size() - 1), new IInputValidator() {

						@Override
						public String isValid(final String newText) {
							if (Pattern.matches("(\\d)+\\-(\\d)+", newText) == false) {
								return "Specify the range in the following way int-int eg. 10-20 ";
							}
							return null;
						}
					});

			final int status = inputDialog.open();

			if (status == 0) {
				final String value = inputDialog.getValue();

				final String[] split = value.split("-");

				subList = new ArrayList(eList).subList(Integer.parseInt(split[0]),
						Integer.parseInt(split[1]) < eList.size() ? Integer.parseInt(split[1]) : (eList.size() - 1));

			}
			else {
				subList = new ArrayList(eList).subList(0, 99);
			}

			return subList;

		}
		return eList;
	}

}
